#include "force.h"
#include "math.h"

#define L 6
#define INIT_ENERGY 4.0

int initial_conditions(Data *data)
{
  double *pos, *vel, *m;
    
  // SIMULATION PARAMTERS
  data->natoms = 1;

  // PER ATOMS PARAMETERS
  data->pos = (double *)malloc( data->natoms*sizeof(double) );
  data->midstep = (double *)malloc( data->natoms*sizeof(double) );
  data->vel = (double *)malloc( data->natoms*sizeof(double) );
  data->f = (double *)malloc( data->natoms*sizeof(double) );
  data->mass = (double *)malloc( data->natoms*sizeof(double) );  

  pos = data->pos;
  vel = data->vel;
  m = data->mass;

  // INITIAL CONDITIONS
  pos[0] = 0;
  vel[0] = sqrt(2.0*INIT_ENERGY);
  m[0] = 1;

  return 0;
}


/* Forces are evaluated on a per atom basis.
 * force_eval is called separately for each atom at each iteration.
 */
int force_eval(Data *data, int i)
{
  int j, k, natoms = data->natoms;
  double *f = data->f, *pos = data->pos, u, sum;
  double **dif_p = data->diftab->dif_p, **mdf_p = data->diftab->mdf_p;

  u = f[i];
  // PIECEWISE FORCE
  if(pos[i] < 0)
    f[i] = -pos[i];
  else if(pos[i] > L)
    f[i] = -pos[i] + L;
  else
    f[i] = 0;

  // BACKWARD DIFFERENCE TABLE UPDATE (FULLSTEP P PORTION ONLY)
  u = (data->dt/(2.0*data->mass[i]))*(f[i] + u); // 1st difference of p
  for(j=NUMSHADOWS; j>0; j--)
    { for(sum=0, k=1; k<j; k++) sum += dif_p[k][i];
      dif_p[j][i] = u - sum; }

  // BACKWARD DIFFERENCE TABLE UPDATE (MIDSTEP P PORTION ONLY)
  u = (data->dt/data->mass[i])*f[i]; // 1st difference of midstep p
  for(j=NUMSHADOWS; j>0; j--)
    { for(sum=0, k=1; k<j; k++) sum += mdf_p[k][i];
      mdf_p[j][i] = u - sum; }
  
  return 0;
}


/* potential_eval is called once per iteration */
int potential_eval(Data *data)
{
  int i, natoms = data->natoms;
  double *pos = data->pos, u;

  data->pe = 0;
  for(i=0; i<natoms; i++)
    {
      // PIECEWISE POTENTIAL
      if(pos[i] < 0)
	data->pe += 0.5*pos[i]*pos[i];
      else if(pos[i] > L)
	data->pe += 0.5*(pos[i] - L)*(pos[i] - L);
      else
	data->pe += 0;
    }

  return 0;
}


/* See comments in truncated.h
 *
 * Truncated shadow Hamiltonians are dependent on the Hamiltonian used.
 *
 * The formulas for interpolated hamiltonians are *not* dependent on the
 *  potential, so those quantities will be valid whatever you change
 *  the potential function to.
 */
int truncated_eval(Data *data)     
{
  trunc_2_eval(data);
  trunc_4_eval(data);
  trunc_6_eval(data);
  trunc_8_eval(data);
  trunc_10_eval(data);
  trunc_12_eval(data);

  return 0;
}

int trunc_2_eval(Data *data)
{

  data->trunc[S2] = 0;
  
  return 0;
}

int trunc_4_eval(Data *data)
{
  data->trunc[S4] = 0;
  
  return 0;
}

int trunc_6_eval(Data *data)
{
  data->trunc[S6] = 0;
  
  return 0;
}

int trunc_8_eval(Data *data)
{
  data->trunc[S8] = 0;

  return 0;
}


int trunc_10_eval(Data *data)
{
  data->trunc[S10] = 0;

  return 0;
}


int trunc_12_eval(Data *data)
{
  data->trunc[S12] = 0;

  return 0;
}


